@@ -1,6 +1,5 @@
 import os
 import copy
-import ppscore
 import numpy as np
 import pandas as pd
 from tqdm import tqdm
@@ -152,8 +151,6 @@ class DataExplorer:
             self.shap()
             print('[AutoML] Generating Feature Ranking Plot')
             self.feature_ranking()
-            print('[AutoML] Predictive Power Score Plot')
-            self.predictive_power_score()
 
         # Mode specific functions
         if self.mode == 'classification':
@@ -233,11 +230,11 @@ class DataExplorer:
             if not os.path.exists(self.folder + 'TimePlots/v{}/'.format(self.version)):
                 os.makedirs(self.folder + 'TimePlots/v{}/'.format(self.version))
 
-            # Set matplot limit
+            # Set pyplot limit
             matplotlib.use('Agg')
             matplotlib.rcParams['agg.path.chunksize'] = 200000
 
-            # Undersample
+            # Under sample
             ind = np.linspace(0, len(self.data) - 1, self.maxSamples).astype('int')
             data, y = self.data.iloc[ind], self.Y.iloc[ind]
 
@@ -271,17 +268,16 @@ class DataExplorer:
 
         # Iterate through features
         for key in tqdm(self.data.keys()):
-            for period in self.seasonPeriods:
-                if self.tag + key + '_v{}.png'.format(self.version) in os.listdir(self.folder + 'Seasonality/'):
-                    continue
-                seasonality = STL(self.data[key], period=period).fit()
-                fig = plt.figure(figsize=[24, 16])
-                plt.plot(range(len(self.data)), self.data[key])
-                plt.plot(range(len(self.data)), seasonality)
-                plt.title(key + ', period=' + str(period))
-                fig.savefig(self.folder + 'Seasonality/' + self.tag + str(period)+'/'+key +
-                            '_v{}.png'.format(self.version), format='png', dpi=300)
-                plt.close()
+            if self.tag + key + '_v{}.png'.format(self.version) in os.listdir(self.folder + 'Seasonality/'):
+                continue
+            seasonality = STL(self.data[key], period=self.seasonPeriods).fit()
+            fig = plt.figure(figsize=[24, 16])
+            plt.plot(range(len(self.data)), self.data[key])
+            plt.plot(range(len(self.data)), seasonality)
+            plt.title(key + ', period=' + str(self.seasonPeriods))
+            fig.savefig(self.folder + 'Seasonality/' + self.tag + str(self.seasonPeriods)+'/'+key +
+                        '_v{}.png'.format(self.version), format='png', dpi=300)
+            plt.close()
 
     def co_linearity(self):
         if self.plotCoLinearity:
@@ -298,7 +294,7 @@ class DataExplorer:
             threshold = 0.95
             fig = plt.figure(figsize=[24, 16])
             plt.title('Co-linearity matrix, threshold {:.2f}'.format(threshold))
-            sns.heatmap(abs(self.data.corr()) < threshold, annot=False, cmap='Greys')
+            plt.imshow(abs(self.data.corr()) < threshold, cmap='gray')
             fig.savefig(self.folder + 'CoLinearity/v{}/'.format(self.version) + self.tag + 'Matrix.png',
                         format='png', dpi=300)
 
@@ -308,7 +304,7 @@ class DataExplorer:
             col_drop = [column for column in upper.columns if any(upper[column] > 0.95)]
             minimal_rep = self.data.drop(self.data[col_drop], axis=1)
             fig = plt.figure(figsize=[24, 16])
-            sns.heatmap(abs(minimal_rep.corr()) < threshold, annot=False, cmap='Greys')
+            plt.imshow(abs(minimal_rep.corr()) < threshold, cmap='gray')
             fig.savefig(self.folder + 'CoLinearity/v{}/'.format(self.version) + self.tag + 'Minimum_Representation.png',
                         format='png', dpi=300)
 
@@ -500,33 +496,34 @@ class DataExplorer:
             results = pd.DataFrame({'x': self.data.keys(), 'score': model.feature_importances_})
             results.to_csv(self.folder + 'Features/v{}/'.format(self.version) + self.tag + 'RF.csv')
 
-    def predictive_power_score(self):
-        if self.plotFeatureImportance:
-            # Create folder
-            if not os.path.exists(self.folder + 'Features/v{}/'.format(self.version)):
-                os.mkdir(self.folder + 'Features/v{}/'.format(self.version))
-
-            # Skip if existing
-            if self.tag + 'PPScore.png' in os.listdir(self.folder + 'Features/v{}/'.format(self.version)):
-                return
-
-            # Calculate PPS
-            data = self.data.copy()
-            if isinstance(self.Y, pd.core.series.Series):
-                data.loc[:, 'target'] = self.Y
-            elif isinstance(self.Y, pd.DataFrame):
-                data.loc[:, 'target'] = self.Y.loc[:, self.Y.keys()[0]]
-            pp_score = ppscore.predictors(data, 'target').sort_values('ppscore')
-
-            # Plot
-            fig, ax = plt.subplots(figsize=[4, 6])
-            plt.subplots_adjust(left=0.5, top=1, bottom=0)
-            ax.spines['right'].set_visible(False)
-            ax.spines['bottom'].set_visible(False)
-            ax.spines['top'].set_visible(False)
-            plt.barh(pp_score['x'][-15:], width=pp_score['ppscore'][-15:], color='#2369ec')
-            fig.savefig(self.folder + 'Features/v{}/'.format(self.version) + self.tag + 'Ppscore.png', format='png', dpi=400)
-            plt.close()
-
-            # Store results
-            pp_score.to_csv(self.folder + 'Features/v{}/pp_score.csv'.format(self.version))
+    # def predictive_power_score(self):
+    #     if self.plotFeatureImportance:
+    #         # Create folder
+    #         if not os.path.exists(self.folder + 'Features/v{}/'.format(self.version)):
+    #             os.mkdir(self.folder + 'Features/v{}/'.format(self.version))
+    #
+    #         # Skip if existing
+    #         if self.tag + 'PPScore.png' in os.listdir(self.folder + 'Features/v{}/'.format(self.version)):
+    #             return
+    #
+    #         # Calculate PPS
+    #         data = self.data.copy()
+    #         if isinstance(self.Y, pd.core.series.Series):
+    #             data.loc[:, 'target'] = self.Y
+    #         elif isinstance(self.Y, pd.DataFrame):
+    #             data.loc[:, 'target'] = self.Y.loc[:, self.Y.keys()[0]]
+    #         pp_score = ppscore.predictors(data, 'target').sort_values('ppscore')
+    #
+    #         # Plot
+    #         fig, ax = plt.subplots(figsize=[4, 6])
+    #         plt.subplots_adjust(left=0.5, top=1, bottom=0)
+    #         ax.spines['right'].set_visible(False)
+    #         ax.spines['bottom'].set_visible(False)
+    #         ax.spines['top'].set_visible(False)
+    #         plt.barh(pp_score['x'][-15:], width=pp_score['ppscore'][-15:], color='#2369ec')
+    #         fig.savefig(self.folder + 'Features/v{}/'.format(self.version) + self.tag + 'Ppscore.png', format='png',
+    #                     dpi=400)
+    #         plt.close()
+    #
+    #         # Store results
+    #         pp_score.to_csv(self.folder + 'Features/v{}/pp_score.csv'.format(self.version))
