@@ -18,6 +18,7 @@ class DataProcesser:
                  outlier_removal: str = 'clip',
                  z_score_threshold: int = 4,
                  version: int = 1,
+                 verbosity: int = 0,
                  ):
         """
         Preprocessing Class. Cleans a dataset into a workable format.
@@ -71,6 +72,7 @@ class DataProcesser:
 
         # Info for Documenting
         self.is_fitted = False
+        self.verbosity = verbosity
         self.removedDuplicateRows = 0
         self.removedDuplicateColumns = 0
         self.removedOutliers = 0
@@ -90,7 +92,8 @@ class DataProcesser:
         -------
         data [pd.DataFrame]: Cleaned input data
         """
-        print('[AutoML] Data Cleaning Started, ({} x {}) samples'.format(len(data), len(data.keys())))
+        if self.verbosity > 0:
+            print('[AutoML] Data Cleaning Started, ({} x {}) samples'.format(len(data), len(data.keys())))
 
         # Clean Keys
         data = clean_keys(data)
@@ -121,7 +124,8 @@ class DataProcesser:
 
         # Finish
         self.is_fitted = True
-        print('[AutoML] Processing completed, ({} x {}) samples returned'.format(len(data), len(data.keys())))
+        if self.verbosity > 0:
+            print('[AutoML] Processing completed, ({} x {}) samples returned'.format(len(data), len(data.keys())))
         return data
 
     def transform(self, data: pd.DataFrame) -> pd.DataFrame:
@@ -191,9 +195,9 @@ class DataProcesser:
         self.int_cols = settings['int_cols'] if 'int_cols' in settings else []
         self.cat_cols = settings['cat_cols'] if 'cat_cols' in settings else []
         self.date_cols = settings['date_cols'] if 'date_cols' in settings else []
-        self.missing_values = settings['missing_values'] if 'num_cols' in settings else []
-        self.outlier_removal = settings['outlier_removal'] if 'num_cols' in settings else []
-        self.z_score_threshold = settings['z_score_threshold'] if 'num_cols' in settings else []
+        self.missing_values = settings['missing_values'] if 'missing_values' in settings else []
+        self.outlier_removal = settings['outlier_removal'] if 'outlier_removal' in settings else []
+        self.z_score_threshold = settings['z_score_threshold'] if 'z_score_threshold' in settings else []
         self._means = None if settings['_means'] is None else pd.read_json(settings['_means'])
         self._stds = None if settings['_stds'] is None else pd.read_json(settings['_stds'])
         self._q1 = None if settings['_q1'] is None else pd.read_json(settings['_q1'])
@@ -213,26 +217,51 @@ class DataProcesser:
             if not self.includeOutput and self.target is not None and self.target in data:
                 data = data.drop(self.target, axis=1)
 
-            # All numeric, floats or ints
-            self.int_cols = data.keys()[data.dtypes == int].tolist()
-            self.float_cols = data.keys()[data.dtypes == float].tolist()
+            # Iterate through keys
+            for key in data.keys():
+                # Integer
+                if pd.api.types.is_integer_dtype(data[key]):
+                    self.int_cols.append(key)
+
+                # Float
+                if pd.api.types.is_float_dtype(data[key]):
+                    self.float_cols.append(key)
+
+                # Datetime
+                if pd.api.types.is_datetime64_any_dtype(data[key]):
+                    self.date_cols.append(key)
+
+                # Booleans
+                if pd.api.types.is_bool_dtype(data[key]):
+                    self.int_cols.append(key)
+
+                # Strings / Objects
+                if pd.api.types.is_object_dtype(data[key]):
+                    is_date = data[key].astype('str').apply(pd.to_datetime, errors='coerce').isna().sum() < 0.3 * \
+                              len(data)
+                    if is_date:
+                        self.date_cols.append(key)
+                    elif data[key].nunique() < 100:
+                        self.cat_cols.append(key)
+                    else:
+                        forced_numeric = pd.to_numeric(data[key], errors='coerce')
+                        if forced_numeric.isna().sum() < len(data) / 25:
+                            self.float_cols.append(key)
+
+            # Set num cols for reverse compatibility
             self.num_cols = self.int_cols + self.float_cols
+
+            # Check if float keys are secretly not integers
             for key in self.float_cols:
                 forced_int = pd.to_numeric(data[key].fillna(0), errors='coerce', downcast='integer')
                 if pd.api.types.is_integer_dtype(forced_int):
                     self.float_cols.remove(key)
                     self.int_cols.append(key)
 
-            # String are either datetime or categorical, we check datetime
-            object_keys = data.keys()[data.dtypes == object]
-            object_is_date = data[object_keys].astype('str').apply(pd.to_datetime, errors='coerce')\
-                .isna().sum() < 0.3 * len(data)
-            self.date_cols = object_keys[object_is_date].tolist()
-            self.cat_cols = object_keys[~object_is_date].tolist()
-
             # Print
-            print(f"[AutoML] Found {len(self.num_cols)} numerical, {len(self.cat_cols)} categorical and "
-                  f"{len(self.date_cols)} datetime columns")
+            if self.verbosity > 0:
+                print(f"[AutoML] Found {len(self.int_cols)} integer, {len(self.float_cols)} float, {len(self.cat_cols)} "
+                      f"categorical and {len(self.date_cols)} datetime columns")
 
         return
 
@@ -261,12 +290,6 @@ class DataProcesser:
         for key in self.float_cols:
             data.loc[:, key] = pd.to_numeric(data[key], errors='coerce', downcast='float')
 
-        # Numeric columns that are not in Float or Integer -- redundancy
-        for key in self.num_cols:
-            if key in self.float_cols + self.int_cols:
-                continue
-            data.loc[:, key] = pd.to_numeric(data[key], errors='coerce')
-
         # Categorical columns
         if fit_categorical:
             data = self._fit_cat_cols(data)
@@ -276,7 +299,7 @@ class DataProcesser:
             data = self._transform_cat_cols(data)
 
         # We need everything to become numeric, so all that is not mentioned will be handled as numeric
-        all_cols = self.float_cols + self.int_cols + self.num_cols + self.date_cols + self.cat_cols
+        all_cols = self.float_cols + self.int_cols + self.date_cols + self.cat_cols + [self.target]
         for key in data.keys():
             if key not in all_cols:
                 data.loc[:, key] = pd.to_numeric(data[key], errors='coerce')
@@ -297,7 +320,8 @@ class DataProcesser:
         """
         Converts categorical variables according to fitted scheme.
         """
-        for key, value in self.dummies.items():
+        for key in self.cat_cols:
+            value = self.dummies[key]
             dummies = [i[len(key) + 1:] for i in value]
             data[value] = np.equal.outer(data[key].values, dummies) * 1
             data = data.drop(key, axis=1)
@@ -326,8 +350,9 @@ class DataProcesser:
         # Note
         self.removedDuplicateColumns = n_columns - len(data.keys())
         self.removedDuplicateRows = n_rows - len(data)
-        print(f'[AutoML] Removed {self.removedDuplicateColumns} duplicate columns and {self.removedDuplicateRows} '
-              f'duplicate rows')
+        if self.verbosity > 0 or (self.removedDuplicateColumns != 0 or self.removedDuplicateRows != 0):
+            print(f'[AutoML] Removed {self.removedDuplicateColumns} duplicate columns and {self.removedDuplicateRows} '
+                  f'duplicate rows')
 
         return data
 
@@ -342,7 +367,8 @@ class DataProcesser:
 
         # Note
         self.removedConstantColumns = columns - len(data.keys())
-        print(f'[AutoML] Removed {self.removedConstantColumns} constant columns.')
+        if self.verbosity > 0 or self.removedConstantColumns != 0:
+            print(f'[AutoML] Removed {self.removedConstantColumns} constant columns.')
 
         return data
 
@@ -354,19 +380,12 @@ class DataProcesser:
         if self.outlier_removal == 'quantiles':
             self._q1 = data[self.num_cols].quantile(0.25)
             self._q3 = data[self.num_cols].quantile(0.75)
-            return (data[self.num_cols] > self._q3).sum().sum() + (data[self.num_cols] < self._q1).sum().sum()
 
         # By z-score
         elif self.outlier_removal == 'z-score':
             self._means = data[self.num_cols].mean(skipna=True, numeric_only=True)
             self._stds = data[self.num_cols].std(skipna=True, numeric_only=True)
             self._stds[self._stds == 0] = 1
-            z_score = (data[self.num_cols] - self._means) / self._stds
-            return (z_score > self.z_score_threshold).sum().sum()
-
-        # By clipping
-        elif self.outlier_removal == 'clip':
-            return (data[self.num_cols] > 1e12).sum().sum() + (data[self.num_cols] < -1e12).sum().sum()
 
     def remove_outliers(self, data: pd.DataFrame, fit: bool = True) -> pd.DataFrame:
         """
@@ -374,22 +393,25 @@ class DataProcesser:
         """
         # Check if needs fitting
         if fit:
-            self.removedOutliers = self.fit_outliers(data)
+            self.fit_outliers(data)
         else:
             assert self.is_fitted, ".remove_outliers() is called with fit=False, yet the object isn't fitted yet."
 
         # With Quantiles
         if self.outlier_removal == 'quantiles':
+            self.removedOutliers = (data[self.num_cols] > self._q3).sum().sum() + (data[self.num_cols] < self._q1).sum().sum()
             data[self.num_cols] = data[self.num_cols].mask(data[self.num_cols] < self._q1)
             data[self.num_cols] = data[self.num_cols].mask(data[self.num_cols] > self._q3)
 
         # With z-score
         elif self.outlier_removal == 'z-score':
             z_score = abs((data[self.num_cols] - self._means) / self._stds)
+            self.removedOutliers = (z_score > self.z_score_threshold).sum().sum()
             data[self.num_cols] = data[self.num_cols].mask(z_score > self.z_score_threshold)
 
         # With clipping
         elif self.outlier_removal == 'clip':
+            self.removedOutliers = (data[self.num_cols] > 1e12).sum().sum() + (data[self.num_cols] < -1e12).sum().sum()
             data[self.num_cols] = data[self.num_cols].clip(lower=-1e12, upper=1e12)
         return data
 
@@ -402,7 +424,8 @@ class DataProcesser:
 
         # Note
         self.imputedMissingValues = data[self.num_cols].isna().sum().sum()
-        print(f'[AutoML] Imputed {self.imputedMissingValues} missing values.')
+        if self.verbosity > 0 or self.imputedMissingValues != 0:
+            print(f'[AutoML] Imputed {self.imputedMissingValues} missing values.')
 
         # Removes all rows with missing values
         if self.missing_values == 'remove_rows':
@@ -492,5 +515,23 @@ class DataProcesser:
             for key in [k for k in keys if k not in data]:
                 data[key] = np.zeros(len(data))
                 imputed.append(key)
-        warnings.warn(f'Imputed {len(imputed)} missing columns! {imputed}')
+        if len(imputed) > 0:
+            warnings.warn(f'Imputed {len(imputed)} missing columns! {imputed}')
         return data
+
+    def prune_features(self, features: list):
+        """
+        For use with AutoML.Pipeline. We practically never use all features. Yet this processor imputes any missing
+        features. This causes redundant operations, memory, and warnings. This function prunes the features to avoid
+        that.
+
+        parameters
+        ----------
+        features [list]: List with required features (NOTE: include required features for extracted)
+        """
+        hash_features = dict([(k, 0) for k in features])
+        self.date_cols = [f for f in self.date_cols if f in hash_features]
+        self.num_cols = [f for f in self.num_cols if f in hash_features]
+        self.int_cols = [f for f in self.int_cols if f in hash_features]
+        self.float_cols = [f for f in self.float_cols if f in hash_features]
+        self.cat_cols = [f for f in self.cat_cols if f in hash_features]
