@@ -1,18 +1,20 @@
 import time
 import copy
-import ppscore
 import warnings
 import itertools
 import numpy as np
 import pandas as pd
-from boruta import BorutaPy
 from tqdm import tqdm
+from boruta import BorutaPy
+from shap import TreeExplainer
 from sklearn.tree import DecisionTreeClassifier
 from sklearn.tree import DecisionTreeRegressor
 from sklearn.ensemble import RandomForestClassifier
 from sklearn.ensemble import RandomForestRegressor
 from sklearn.cluster import MiniBatchKMeans
 from Amplo.Utils import clean_keys
+from Amplo.Classifiers import CatBoostClassifier
+from Amplo.Regressors import CatBoostRegressor
 
 
 class FeatureProcesser:
@@ -22,6 +24,8 @@ class FeatureProcesser:
                  max_diff: int = 2,
                  date_cols: list = None,
                  information_threshold: float = 0.99,
+                 selection_cutoff: float = 0.85,
+                 selection_increment: float = 0.005,
                  extract_features: bool = True,
                  mode: str = 'classification',
                  timeout: int = 900,
@@ -87,11 +91,14 @@ class FeatureProcesser:
         self._means = None
         self._stds = None
         self._centers = None
+        self.featureImportance = {}
 
         # Parameters
         self.maxLags = max_lags
         self.maxDiff = max_diff
         self.informationThreshold = information_threshold
+        self.selectionCutoff = selection_cutoff
+        self.selectionIncrement = selection_increment
         self.extractFeatures = extract_features
         self.verbosity = verbosity
 
@@ -140,8 +147,9 @@ class FeatureProcesser:
             self._add_lagged_features()
 
         # Select
-        self.featureSets['PPS'] = self._sel_predictive_power_score()
+        # self.featureSets['PPS'] = self._sel_predictive_power_score()
         self.featureSets['RFT'], self.featureSets['RFI'] = self._sel_gini_impurity()
+        self.featureSets['ShapThreshold'], self.featureSets['ShapIncrement'] = self._sel_shap()
 
         # Set fitted
         self.is_fitted = True
@@ -164,30 +172,7 @@ class FeatureProcesser:
 
         # Get original features
         features = self.featureSets[feature_set]
-        self.linearFeatures = [k for k in features if '__sub__' in k or '__add__' in k]
-        self.crossFeatures = [k for k in features if '__x__' in k or '__d__' in k]
-        self.trigonometricFeatures = [k for k in features if 'sin__' in k or 'cos__' in k]
-        self.inverseFeatures = [k for k in features if 'inv__' in k]
-        self.kMeansFeatures = [k for k in features if 'dist__' in k]
-        self.diffFeatures = [k for k in features if '__diff__' in k]
-        self.laggedFeatures = [k for k in features if '__lag__' in k]
-        self.datetimeFeatures = [k for k in features if '__dt__' in k]
-        original_features = [k for k in features if '__' not in k]
-
-        # Fill missing features for normalization
-        required = copy.copy(original_features)
-        required += list(itertools.chain.from_iterable([s.split('__')[::2] for s in self.linearFeatures]))
-        required += list(itertools.chain.from_iterable([s.split('__')[::2] for s in self.crossFeatures]))
-        required += [s.split('__')[1] for s in self.trigonometricFeatures]
-        required += [s[5:] for s in self.inverseFeatures]
-        required += [s.split('__diff__')[0] for s in self.diffFeatures]
-        required += [s.split('__lag__')[0] for s in self.laggedFeatures]
-        required += [s.split('__dt__')[0] for s in self.datetimeFeatures]
-        if len(self.kMeansFeatures) != 0:
-            required += list(self._centers.keys())
-
-        # Remove duplicates from required
-        required = list(set(required))
+        required = self.get_required_features(features)
 
         # Impute missing keys
         missing_keys = [k for k in required if k not in data.keys()]
@@ -245,9 +230,10 @@ class FeatureProcesser:
             'laggedFeatures': self.laggedFeatures,
             'diffFeatures': self.diffFeatures,
             'featureSets': self.featureSets,
-            '_means': self._means.to_json(),
-            '_stds': self._stds.to_json(),
-            '_centers': self._centers.to_json(),
+            '_means': '[]' if self._means is None else self._means.to_json(),
+            '_stds': '[]' if self._stds is None else self._stds.to_json(),
+            '_centers': '[]' if self._centers is None else self._centers.to_json(),
+            'featureImportance': self.featureImportance
         }
 
     def load_settings(self, settings: dict) -> None:
@@ -265,7 +251,9 @@ class FeatureProcesser:
         self._means = pd.read_json(settings['_means'], typ='series') if 'v' in settings else []
         self._stds = pd.read_json(settings['_stds'], typ='series') if '_stds' in settings else []
         self._centers = pd.read_json(settings['_centers']) if '_centers' in settings else []
+        self.featureImportance = settings['featureImportance'] if 'featureImportance' in settings else {}
         self.is_fitted = True
+        self.verbosity = 0
 
     def _clean_set(self, x: pd.DataFrame, y: pd.Series) -> None:
         """
@@ -600,7 +588,7 @@ class FeatureProcesser:
 
         scores = {}
         for i, key in enumerate(self.originalInput.keys()):
-            inv_feature = 1 / self.x[key]
+            inv_feature = (1 / self.x[key]).clip(lower=-1e12, upper=1e12)
             score = self._analyse_feature(inv_feature)
             if self._accept_feature(score):
                 scores['inv__' + key] = score
@@ -614,7 +602,7 @@ class FeatureProcesser:
         # Add features
         for k in self.inverseFeatures:
             key = k[5:]
-            self.x[k] = 1 / self.x[key]
+            self.x[k] = (1 / self.x[key]).clip(lower=-1e12, upper=1e12)
 
         # Store
         if self.verbosity > 0:
@@ -757,27 +745,27 @@ class FeatureProcesser:
         if self.verbosity > 0:
             print('[AutoML] Added {} lagged features'.format(len(self.laggedFeatures)))
 
-    def _sel_predictive_power_score(self):
-        """
-        Calculates the Predictive Power Score (https://github.com/8080labs/ppscore)
-        Assymmetric correlation based on single decision trees trained on 5.000 samples with 4-Fold validation.
-        """
-        if self.verbosity > 0:
-            print('[AutoML] Determining features with PPS')
-
-        # Copy data
-        data = self.x.copy()
-        data['target'] = self.y.copy()
-
-        # Get Predictive Power Score
-        pp_score = ppscore.predictors(data, "target")
-
-        # Select columns
-        pp_cols = pp_score['x'][pp_score['ppscore'] != 0].to_list()
-
-        if self.verbosity > 0:
-            print('[AutoML] Selected {} features with Predictive Power Score'.format(len(pp_cols)))
-        return pp_cols
+    # def _sel_predictive_power_score(self):
+    #     """
+    #     Calculates the Predictive Power Score (https://github.com/8080labs/ppscore)
+    #     Asymmetric correlation based on single decision trees trained on 5.000 samples with 4-Fold validation.
+    #     """
+    #     if self.verbosity > 0:
+    #         print('[AutoML] Determining features with PPS')
+    #
+    #     # Copy data
+    #     data = self.x.copy()
+    #     data['target'] = self.y.copy()
+    #
+    #     # Get Predictive Power Score
+    #     pp_score = ppscore.predictors(data, "target")
+    #
+    #     # Select columns
+    #     pp_cols = pp_score['x'][pp_score['ppscore'] != 0].to_list()
+    #
+    #     if self.verbosity > 0:
+    #         print('[AutoML] Selected {} features with Predictive Power Score'.format(len(pp_cols)))
+    #     return pp_cols
 
     def _sel_gini_impurity(self):
         """
@@ -785,13 +773,14 @@ class FeatureProcesser:
         Symmetric correlation based on multiple features and multiple trees ensemble
         """
         if self.verbosity > 0:
-            print('[AutoML] Determining features with RF')
+            print('[AutoML] Analysing features with a Random Forest')
 
         # Set model
+        rs = np.random.RandomState(seed=236868)
         if self.mode == 'regression':
-            rf = RandomForestRegressor().fit(self.x, self.y)
+            rf = RandomForestRegressor(random_state=rs).fit(self.x, self.y)
         elif self.mode == 'classification' or self.mode == 'multiclass':
-            rf = RandomForestClassifier().fit(self.x, self.y)
+            rf = RandomForestClassifier(random_state=rs).fit(self.x, self.y)
         else:
             raise ValueError('Method not implemented')
 
@@ -800,30 +789,103 @@ class FeatureProcesser:
         sfi = fi.sum()
         ind = np.flip(np.argsort(fi))
 
+        # Add to class attribute
+        self.featureImportance['rf'] = (self.x.keys()[ind].to_list(), fi[ind].tolist())
+
         # Info Threshold
-        ind_keep = [ind[i] for i in range(len(ind)) if fi[ind[:i]].sum() <= 0.85 * sfi]
+        ind_keep = [ind[i] for i in range(len(ind)) if fi[ind[:i]].sum() <= self.selectionCutoff * sfi]
         threshold = self.x.keys()[ind_keep].to_list()
 
         # Info increment
-        ind_keep = [ind[i] for i in range(len(ind)) if fi[i] > sfi / 200]
+        ind_keep = [ind[i] for i in range(len(ind)) if fi[i] > sfi * self.selectionIncrement]
         increment = self.x.keys()[ind_keep].to_list()
 
         if self.verbosity > 0:
-            print('[AutoML] Selected {} features with 85% RF threshold'.format(len(threshold)))
-            print('[AutoML] Selected {} features with 0.5% RF increment'.format(len(increment)))
+            print(f'[AutoML] Selected {len(threshold)} features with {self.selectionCutoff * 100}% RF threshold')
+            print(f'[AutoML] Selected {len(increment)} features with {self.selectionIncrement * 100}% RF increment')
         return threshold, increment
 
-    def _borutapy(self):
+    def _sel_shap(self):
+        """
+        Calculates Shapely values, which can be used as a measure of feature importance.
+        """
         if self.verbosity > 0:
-            print('[AutoML] Determining features with Boruta')
-        rf = None
+            print('[AutoML] Analysing features with Shapely Additive Explanations')
+        # Get base model
+        base = None
         if self.mode == 'regression':
-            rf = RandomForestRegressor()
+            base = CatBoostRegressor()
         elif self.mode == 'classification' or self.mode == 'multiclass':
-            rf = RandomForestClassifier()
-        selector = BorutaPy(rf, n_estimators='auto', verbose=0)
-        selector.fit(self.x.to_numpy(), self.y.to_numpy())
-        bp_cols = self.x.keys()[selector.support_].to_list()
+            base = CatBoostClassifier()
+
+        # Fit
+        base.fit(self.x, self.y)
+
+        # Get SHAP values
+        explainer = TreeExplainer(base.model)
+        values = np.mean(np.abs(explainer.shap_values(self.x, self.y)), axis=0)
+        values_sum = np.sum(values)
+        ind = np.flip(np.argsort(values))
+
+        # Add to class attribute
+        self.featureImportance['shap'] = (self.x.keys()[ind].to_list(), values[ind].tolist())
+
+        # Threshold
+        ind_keep = [ind[i] for i in range(len(ind)) if values[ind[:i]].sum() <= self.selectionCutoff * values_sum]
+        threshold = self.x.keys()[ind_keep].to_list()
+
+        # Increment
+        ind_keep = [ind[i] for i in range(len(ind)) if values[ind[i]] > values_sum * self.selectionIncrement * 10]
+        increment = self.x.keys()[ind_keep].to_list()
+
         if self.verbosity > 0:
-            print('[AutoML] Selected {} features with Boruta'.format(len(bp_cols)))
-        return bp_cols
+            print(f'[AutoML] Selected {len(threshold)} features with {self.selectionCutoff * 100}% Shap threshold')
+            print(f"[AutoML] Selected {len(increment)} features with {self.selectionIncrement * 1000}% Shap increment")
+
+        return threshold, increment
+
+    # Deprecated
+    # def _borutapy(self):
+    #     if self.verbosity > 0:
+    #         print('[AutoML] Analysing features with Boruta')
+    #     rf = None
+    #     if self.mode == 'regression':
+    #         rf = RandomForestRegressor()
+    #     elif self.mode == 'classification' or self.mode == 'multiclass':
+    #         rf = RandomForestClassifier()
+    #     selector = BorutaPy(rf, n_estimators='auto', verbose=0)
+    #     selector.fit(self.x.to_numpy(), self.y.to_numpy())
+    #     bp_cols = self.x.keys()[selector.support_].to_list()
+    #     if self.verbosity > 0:
+    #         print('[AutoML] Selected {} features with Boruta'.format(len(bp_cols)))
+    #     return bp_cols
+
+    def get_required_features(self, features: list):
+        """
+        Generate a list of required input features from a list of extracted features.
+        """
+        # Identify type of features and set class variables
+        self.linearFeatures = [k for k in features if '__sub__' in k or '__add__' in k]
+        self.crossFeatures = [k for k in features if '__x__' in k or '__d__' in k]
+        self.trigonometricFeatures = [k for k in features if 'sin__' in k or 'cos__' in k]
+        self.inverseFeatures = [k for k in features if 'inv__' in k]
+        self.kMeansFeatures = [k for k in features if 'dist__' in k]
+        self.diffFeatures = [k for k in features if '__diff__' in k]
+        self.laggedFeatures = [k for k in features if '__lag__' in k]
+        self.datetimeFeatures = [k for k in features if '__dt__' in k]
+        original_features = [k for k in features if '__' not in k]
+
+        # Fill missing features for normalization
+        required = copy.copy(original_features)
+        required += list(itertools.chain.from_iterable([s.split('__')[::2] for s in self.linearFeatures]))
+        required += list(itertools.chain.from_iterable([s.split('__')[::2] for s in self.crossFeatures]))
+        required += [s.split('__')[1] for s in self.trigonometricFeatures]
+        required += [s[5:] for s in self.inverseFeatures]
+        required += [s.split('__diff__')[0] for s in self.diffFeatures]
+        required += [s.split('__lag__')[0] for s in self.laggedFeatures]
+        required += [s.split('__dt__')[0] for s in self.datetimeFeatures]
+        if len(self.kMeansFeatures) != 0:
+            required += list(self._centers.keys())
+
+        # Remove duplicates from required
+        return list(set(required))
