@@ -76,7 +76,7 @@ class OptunaGridSearch:
         # Regression models
         elif type(self.model).__name__ == 'DecisionTreeRegressor':
             return {
-                'criterion': trial.suggest_categorical('criterion', ['mse', 'friedman_mse', 'mae', 'poisson']),
+                'criterion': trial.suggest_categorical('criterion', ['squared_error', 'friedman_mse', 'absolute_error', 'poisson']),
                 'max_depth': trial.sugest_int('max_depth', 3, min(25, int(np.log2(self.samples)))),
             }
         elif type(self.model).__name__ == 'BaggingRegressor':
@@ -120,7 +120,7 @@ class OptunaGridSearch:
         elif type(self.model).__name__ == 'RandomForestRegressor':
             return {
                 'n_estimators': trial.suggest_int('n_estimators', 50, 1000),
-                'criterion': trial.suggest_categorical('criterion', ['mse', 'mae']),
+                'criterion': trial.suggest_categorical('criterion', ['squared_error', 'absolute_error']),
                 'max_depth': trial.suggest_int('max_depth', 3, min(15, int(np.log2(self.samples)))),
                 'max_features': trial.suggest_categorical('max_features', ['auto', 'sqrt']),
                 'min_samples_split': trial.suggest_int('min_samples_split', 2, 50),
@@ -157,6 +157,7 @@ class OptunaGridSearch:
                 'colsample_bytree': trial.suggest_uniform('colsample_bytree', 0, 1),
                 'reg_alpha': trial.suggest_uniform('reg_alpha', 0, 1),
                 'reg_lambda': trial.suggest_uniform('reg_lambda', 0, 1),
+                'callbacks': [optuna.integration.LightGBMPruningCallback(trial, "mean_absolute_error", "valid_1")],
             }
 
         # Classifiers
@@ -241,12 +242,14 @@ class OptunaGridSearch:
                 "boosting_type": "gbdt",
                 "lambda_l1": trial.suggest_loguniform("lambda_l1", 1e-8, 10.0),
                 "lambda_l2": trial.suggest_loguniform("lambda_l2", 1e-8, 10.0),
-                "num_leaves": trial.suggest_int("num_leaves", 2, 256),
+                "num_leaves": trial.suggest_int("num_leaves", 10, 5000),
+                "max_depth": trial.suggest_int("max_depth", 5, 20),
+                "min_data_in_leaf": trial.suggest_int("min_data_in_leaf", 1, min(1000, int(self.samples / 10))),
+                "min_gain_to_split": trial.suggest_uniform("min_gain_to_split", 0, 5),
                 "feature_fraction": trial.suggest_uniform("feature_fraction", 0.4, 1.0),
                 "bagging_fraction": trial.suggest_uniform("bagging_fraction", 0.4, 1.0),
                 "bagging_freq": trial.suggest_int("bagging_freq", 1, 7),
-                "min_data_in_leaf": trial.suggest_int("min_data_in_leaf", 1, min(1000, int(self.samples / 10))),
-                'callbacks': optuna.integration.LightGBMPruningCallback(trial, "neg_log_loss", "valid_1"),
+                'callbacks': [optuna.integration.LightGBMPruningCallback(trial, "neg_log_loss", "valid_1")],
             }
         else:
             # Raise error if nothing is returned
@@ -268,7 +271,7 @@ class OptunaGridSearch:
         self.x, self.y = x, y
 
         # Set up study
-        study = optuna.create_study(sampler=optuna.samplers.TPESampler(), direction='maximize')
+        study = optuna.create_study(sampler=optuna.samplers.TPESampler(seed=236868), direction='maximize')
         study.optimize(self.objective, timeout=self.timeout, n_trials=self.nTrials)
 
         # Parse results
@@ -283,6 +286,7 @@ class OptunaGridSearch:
             'mean_time': optuna_results['user_attrs_mean_time'],
             'std_time': optuna_results['user_attrs_std_time']
         })
+
         return results
 
     def objective(self, trial):
@@ -306,7 +310,13 @@ class OptunaGridSearch:
             scores.append(self.scoring(model, xv, yv))
             times.append(time.time() - t_start)
 
+        # Set manual metrics
         trial.set_user_attr('mean_time', np.mean(times))
         trial.set_user_attr('std_time', np.std(times))
         trial.set_user_attr('std_value', np.std(scores))
+
+        # Stop trail (avoid overwriting)
+        if trial.number == self.nTrials:
+            trial.study.stop()
+
         return np.mean(scores)
