@@ -2,7 +2,7 @@ import unittest
 import numpy as np
 import pandas as pd
 from sklearn.datasets import load_iris
-from sklearn.datasets import load_boston
+from sklearn.datasets import fetch_california_housing
 from Amplo.AutoML import DataProcesser
 from Amplo.Utils import check_dataframe_quality
 
@@ -13,7 +13,7 @@ class TestDataProcessing(unittest.TestCase):
     def setUpClass(cls):
         cls.classification, y = load_iris(return_X_y=True, as_frame=True)
         cls.classification['target'] = y
-        x, y = load_boston(return_X_y=True)
+        x, y = fetch_california_housing(return_X_y=True)
         cls.regression = pd.DataFrame({'target': y})
         for i in range(x.shape[1]):
             cls.regression['feature_{}'.format(i)] = x[:, i]
@@ -129,9 +129,7 @@ class TestDataProcessing(unittest.TestCase):
 
         # Quantiles
         dp = DataProcesser(outlier_removal='quantiles', target='target')
-        print(x.quantile(0.75))
         xt = dp.fit_transform(x)
-        print(dp._q3, xt)
         assert xt.max().max() < 1e15, "Outlier not removed"
         assert not xt.isna().any().any(), "NaN found"
         assert dp.transform(pd.DataFrame({'a': [2], 'b': [-2]})).max().max() == 0
@@ -169,3 +167,11 @@ class TestDataProcessing(unittest.TestCase):
         dp2.load_settings(settings)
         xt2 = dp2.transform(pd.DataFrame({'a': ['a', 'b'], 'b': [1, 2]}))
         assert np.allclose(pd.DataFrame({'b': [1.0, 2.0], 'a_b': [0, 1], 'a_c': [0, 0]}).values, xt2.values)
+
+    def test_pruner(self):
+        x = pd.DataFrame({'a': ['a', 'b', 'c', 'b', 'c', 'a'], 'b': [1, 1, 1, 1, 1, 1]})
+        dp = DataProcesser()
+        dp.fit_transform(x)
+        dp.prune_features(['b'])
+        assert dp.int_cols == ['b']
+        assert dp.cat_cols == []
